#!/usr/bin/perl

#
# Copyright (C) 2010, 2011, 2012 FromDual GmbH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use warnings;

require v5.6.0;

use Cwd 'abs_path';

my $lMyName = Cwd::abs_path(__FILE__);
my $lMyNameBase = basename($lMyName, '.pl');
my $basedir = dirname($lMyName);

BEGIN {
  use File::Basename;
  unshift(@INC, dirname($0). '/lib');
}

# What about lib::FromDualMySQLagent?
use FromDualMySQLagent ':stooges';
use uploadData;

my $file = $ARGV[0];
my $lConfFile = '';
if ( defined($file) && ($file ne '') ) {
  $lConfFile = $file;
}
else {
  $lConfFile = $basedir . '/etc/' . $lMyNameBase . '.conf';
}

our $gDomain = 'FromDual.MySQL';

my @configuration = &FromDualMySQLagent::readConfigurationFile($lConfFile);
my @sections = &FromDualMySQLagent::getSections(@configuration);
# Read the default section here
my %defaults = &FromDualMySQLagent::getDefaultParameter(\@configuration, 'default');

my $lDefaultsLogFile = $defaults{'LogFile'};
my $lDefaultsDebug   = $defaults{'Debug'};

# Touch logfile
open LOG, ">>" . $lDefaultsLogFile or die $!;
close(LOG);

&FromDualMySQLagent::setAgentLock($lDefaultsDebug, $lDefaultsLogFile);

if ( $lDefaultsDebug >= INFO ) {
  &FromDualMySQLagent::log($lDefaultsLogFile, INFO, "FromDual Performance Monitor for MySQL run started.");
  &FromDualMySQLagent::log($lDefaultsLogFile, INFO, "Read configuratoin from $lConfFile");
  &FromDualMySQLagent::log($lDefaultsLogFile, INFO, 'Sections found: ' . join(', ', @sections));
  &FromDualMySQLagent::log($lDefaultsLogFile, INFO, "Reading default section");
}
if ( $lDefaultsDebug >= DBG ) {

  foreach my $p ( keys %defaults) {

    # This would be possibly a security problem...
    if ( $p eq 'Password' ) {
      &FromDualMySQLagent::log($lDefaultsLogFile, DBG, "  $p - ********");
    }
    else {
      &FromDualMySQLagent::log($lDefaultsLogFile, DBG, "  $p - $defaults{$p}");
    }
  }
}

foreach my $section ( @sections ) {

  if ( $lDefaultsDebug >= INFO ) { &FromDualMySQLagent::log($lDefaultsLogFile, INFO, "Processing section $section now"); }

  # skip default section, we read it already above
  if ( $section ne 'default' ) {

    my %parameter = &FromDualMySQLagent::getParameter(\@configuration, $section);

    if ( $parameter{'Debug'} == DBG ) {

      &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, "The combined (default + section) parameters for section $section are:");
      foreach my $p ( keys %parameter ) {

        # This would be possibly a security problem...
        if ( $p eq 'Password' ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, "  $p - ********");
        }
        else {
          &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, "  $p - $parameter{$p}");
        }
      }
    }

    # Skip section if it is disabled
    if ( $parameter{'Disabled'} eq 'true' ) {

      if ( $parameter{'Debug'} >= INFO ) {
        &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Skipping section: $section because it is disabled.");
      }
      next;
    }

    if ( $parameter{'Debug'} >= INFO ) { &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Modules for section $section: " . $parameter{'Modules'}); }

    my @modules = split(/\s+/, $parameter{'Modules'});
    foreach my $module ( @modules ) {

      # FromDual.MySQL.xxx
      $parameter{'Domain'} = $gDomain . '.' . $module;

      if ( ($parameter{'Debug'} >= INFO) ) { &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "  Processing module $module for section $section now..."); }

      if ( ($module eq 'aria')
        || ($module eq 'drbd')
        || ($module eq 'galera')
        || ($module eq 'innodb')
        || ($module eq 'master')
        || ($module eq 'memcache')
        || ($module eq 'mpm')
        || ($module eq 'myisam')
        || ($module eq 'mysql')
        || ($module eq 'ndb')
        || ($module eq 'pbxt')
        || ($module eq 'process')
        || ($module eq 'server')
        || ($module eq 'slave')
         ) {

        # eval {
        #   require FromDualMySQLmysql;
        # };
        # Call module lib::FromDualMySQLxxx
        my $class = 'FromDualMySQL' . $module;
        eval "require $class";

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQL" . $module . " does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, $key);
          }
        }
        else {

          our %gParameter = %parameter;
          # &FromDualMySQLmysql::processMysqlInformation(\%parameter);
          my $cmd = "&FromDualMySQL" . $module . "::process". ucfirst($module) . "Information(\%parameter);";
          # print "$cmd\n";
          eval $cmd;
        }
      }
      else {
        &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module $module does not exist.");
      }
    }
  }
}

&FromDualMySQLagent::removeAgentLock($lDefaultsDebug, $lDefaultsLogFile);

print "1\n";
exit 0;
