package FromDualMySQLinnodb;

#
# Copyright (C) 2010, 2011, 2012 FromDual GmbH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use warnings;

use FromDualMySQLagent ':stooges';

sub processInnodbInformation
{
  if ( $main::gParameter{'Debug'} >= INFO ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, INFO, '    ' . (caller(0))[3]); }

  my %hGlobalVariables;
  my %hGlobalStatus;
  my %hInnodbStatus;

  my @aGlobalVariablesToSend = (
    'ignore_builtin_innodb'
  , 'innodb_buffer_pool_instances'
  , 'innodb_buffer_pool_size'
  , 'innodb_file_format'
  , 'innodb_file_per_table'
  , 'innodb_flush_log_at_trx_commit'
  , 'innodb_force_recovery'
  , 'innodb_io_capacity'
  , 'innodb_log_buffer_size'
  , 'innodb_log_file_size'
);

  my @aGlobalStatusToSend = (
    'Innodb_buffer_pool_pages_data'
  , 'Innodb_buffer_pool_pages_dirty'
  , 'Innodb_buffer_pool_pages_free'
  , 'Innodb_buffer_pool_pages_misc'
  , 'Innodb_buffer_pool_pages_total'
  , 'Innodb_buffer_pool_reads'
  , 'Innodb_buffer_pool_read_requests'
  , 'Innodb_data_fsyncs'
  , 'Innodb_data_reads'
  , 'Innodb_data_writes'
  , 'Innodb_data_read'
  , 'Innodb_data_written'
  , 'Innodb_row_lock_time'
  , 'Innodb_row_lock_waits'
  , 'Innodb_rows_deleted'
  , 'Innodb_rows_inserted'
  , 'Innodb_rows_read'
  , 'Innodb_rows_updated'
  );

  my @aInnodbStatusToSend = (
    'Innodb_compress_time'
  , 'Innodb_hash_node_size'
  , 'Innodb_hash_seaches'
  , 'Innodb_last_checkpoint_at'
  , 'Innodb_log_flushed_up_to'
  , 'Innodb_log_sequence_number'
  , 'Innodb_mutex_os_waits'
  , 'Innodb_mutex_spin_rounds'
  , 'Innodb_mutex_spin_waits'
  , 'Innodb_non_hash_searches'
  , 'Innodb_open_read_views_inside_innodb'
  , 'Innodb_pages_flushed_up_to'
  , 'Innodb_queries_in_queue'
  , 'Innodb_queries_inside_innodb'
  , 'Innodb_rows_locked'
  , 'Innodb_rows_modified'
  , 'Innodb_trx_committing'
  , 'Innodb_trx_history_list_length'
  , 'Innodb_trx_lock_memory'
  , 'Innodb_trx_lock_wait'
  , 'Innodb_trx_rolling_back'
  , 'Innodb_trx_running'
  , 'Innodb_uncompress_time'
);

  &FromDualMySQLagent::initValues(\%hGlobalVariables, \@aGlobalVariablesToSend);
  &FromDualMySQLagent::initValues(\%hGlobalStatus, \@aGlobalStatusToSend);
  &FromDualMySQLagent::initValues(\%hInnodbStatus, \@aInnodbStatusToSend);

  # Set defaults
  $hGlobalVariables{'innodb_io_capacity'}           = 200;
  $hGlobalVariables{'innodb_buffer_pool_instances'} = 1;
  $hGlobalVariables{'innodb_file_per_table'}        = 'Antelope';

  my $dbh = &FromDualMySQLagent::getDatabaseConnection(%main::gParameter);

  if ( ! defined($dbh) ) {
    if ( $main::gParameter{'Debug'} >= ERR ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, DBG, '    Database connection failed.'); }
    return -1;
  }

  &FromDualMySQLagent::getGlobalVariables($dbh, \%hGlobalVariables);
  &FromDualMySQLagent::getGlobalStatus($dbh, \%hGlobalStatus);
  &FromDualMySQLagent::getInnodbStatus($dbh, \%hInnodbStatus);

  &FromDualMySQLagent::releaseDatabaseConnection($dbh);

  # Do some post calculations

  &FromDualMySQLagent::sendData(\%hGlobalVariables, \@aGlobalVariablesToSend);
  &FromDualMySQLagent::sendData(\%hGlobalStatus, \@aGlobalStatusToSend);
  &FromDualMySQLagent::sendData(\%hInnodbStatus, \@aInnodbStatusToSend);
}

1;
__END__

