package FromDualMySQLmysql;

#
# Copyright (C) 2010, 2011, 2012 FromDual GmbH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use warnings;

use FromDualMySQLagent ':stooges';

sub processMysqlInformation
{
  if ( $main::gParameter{'Debug'} >= INFO ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, INFO, '    ' . (caller(0))[3]); }

  my %hGlobalVariables;
  my %hGlobalStatus;

  my @aGlobalVariablesToSend = (
    'alive'
  , 'character_set_server'
  , 'datadir'
  , 'default_storage_engine'
  , 'event_scheduler'
  , 'foreign_key_checks'
  , 'general_log'
  , 'log_output'
  , 'log_queries_not_using_indexes'
  , 'log_slave_updates'
  , 'log_warnings'
  , 'long_query_time'
  , 'max_connections'
  , 'max_user_connections'
  , 'max_heap_table_size'
  , 'query_cache_size'
  , 'query_cache_type'
  , 'skip_networking'
  , 'slow_query_log'
  , 'sql_mode'
  , 'storage_engine'
  , 'table_cache'
  , 'table_definition_cache'
  , 'table_open_cache'
  , 'thread_cache_size'
  , 'thread_stack'
  , 'tmp_table_size'
  , 'version'
  );

  my @aGlobalStatusToSend = (
    'Aborted_clients'
  , 'Aborted_connects'
  , 'Bytes_received'
  , 'Bytes_sent'
  , 'Com_call_proc'
  , 'Com_delete'
  , 'Com_delete_multi'
  , 'Com_insert'
  , 'Com_insert_select'
  , 'Com_replace'
  , 'Com_replace_select'
  , 'Com_select'
  , 'Com_update'
  , 'Com_update_multi'
  , 'Connections'
  , 'Created_tmp_disk_tables'
  , 'Created_tmp_tables'
  , 'Handler_delete'
  , 'Handler_read_first'
  , 'Handler_read_key'
  , 'Handler_read_next'
  , 'Handler_read_prev'
  , 'Handler_read_rnd'
  , 'Handler_read_rnd_next'
  , 'Handler_update'
  , 'Handler_write'
  , 'Max_used_connections'
  , 'Opened_tables'
  , 'Opened_table_definitions'
  , 'Open_tables'
  , 'Open_table_definitions'
  , 'Qcache_free_blocks'
  , 'Qcache_free_memory'
  , 'Qcache_hits'
  , 'Qcache_inserts'
  , 'Qcache_lowmem_prunes'
  , 'Qcache_not_cached'
  , 'Qcache_queries_in_cache'
  , 'Qcache_total_blocks'
  , 'Queries'
  , 'Questions'
  , 'Select_full_join'
  , 'Select_full_range_join'
  , 'Select_range_check'
  , 'Select_scan'
  , 'Slow_queries'
  , 'Sort_merge_passes'
  , 'Sort_range'
  , 'Sort_rows'
  , 'Sort_scan'
  , 'Table_locks_immediate'
  , 'Table_locks_waited'
  , 'Threads_cached'
  , 'Threads_connected'
  , 'Threads_created'
  , 'Threads_running'
  , 'Uptime'
  , 'Access_denied_errors'
  , 'Binlog_bytes_written'
  , 'Handler_read_last'
  , 'Binlog_cache_disk_use'
  , 'Binlog_cache_use'
  , 'Created_tmp_files'
  , 'Opened_files'
  );

  &FromDualMySQLagent::initValues(\%hGlobalVariables, \@aGlobalVariablesToSend);
  &FromDualMySQLagent::initValues(\%hGlobalStatus, \@aGlobalStatusToSend);

  # This should be done automatically by initValues
  # $hGlobalVariables{'alive'} = 0;
  my $dbh = &FromDualMySQLagent::getDatabaseConnection(%main::gParameter);

  if ( ! defined($dbh) ) {
    if ( $main::gParameter{'Debug'} >= ERR ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, DBG, '    Database connection failed.'); }
    return -1;
  }

  # &FromDualMySQLagent::getGlobalVariables(\%main::gParameter, $dbh, \%hGlobalVariables);
  &FromDualMySQLagent::getGlobalVariables($dbh, \%hGlobalVariables);
  &FromDualMySQLagent::getGlobalStatus($dbh, \%hGlobalStatus);

  &FromDualMySQLagent::releaseDatabaseConnection($dbh);

  # Do some post calculations

  my $count = keys %hGlobalVariables;
  if ( $count > 0 ) {
    $hGlobalVariables{'alive'} = 1;

    # Set it to a value otherwise we get an error message!
    if ( $hGlobalVariables{'sql_mode'} eq '' ) {
      $hGlobalVariables{'sql_mode'} = 'none';
    }
  }

  &FromDualMySQLagent::sendData(\%hGlobalVariables, \@aGlobalVariablesToSend);
  &FromDualMySQLagent::sendData(\%hGlobalStatus, \@aGlobalStatusToSend);
}

1;
__END__

