package uploadData;

#
# Copyright (C) 2010, 2011, 2012 FromDual GmbH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.
# If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use warnings;

# check if HTTP::Request::Common and LWP::Simple; exists and if not do a warning!!!

# http://www.perl.com/pub/2002/08/20/perlandlwp.html
# http://perl.active-venture.com/lib/HTTP/Request/Common.html
# http://stackoverflow.com/questions/3836526/how-do-i-send-post-data-with-lwp

use LWP 5.64;
use HTTP::Request::Common qw{ POST };
use Getopt::Long;
use File::Basename;

use constant DBG  => 4;
use constant INFO => 3;
use constant WARN => 2;
use constant ERR  => 1;

sub uploadData
{
  my $rc = 0;

  if ( $main::gParameter{'Debug'} >= INFO ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, INFO, '       ' . (caller(0))[3]); }

  my $browser = LWP::UserAgent->new;

  if ( ! -r $main::gParameter{'CacheFile'} ) {
    $rc = 1404;
    &FromDualMySQLagent::log($main::gParameter{'LogFile'}, ERR, "Cannot read file $main::gParameter{'CacheFile'} (rc=$rc).");
    return $rc;
  }

  my $response = $browser->request(POST $main::gParameter{'Url'},
    Content_Type => 'form-data',
    Content      => [ hash      => $main::gParameter{'Hash'},
                      filename  => [$main::gParameter{'CacheFile'}],
                    ]); # ->as_string()

  if ( ! $response->is_success ) {
    $rc = 1401;
    &FromDualMySQLagent::log($main::gParameter{'LogFile'}, ERR, '      ' . $main::gParameter{'Url'} . " error: " . $response->status_line . " (rc=$rc)");
    return $rc;
  }

  if ( $response->content_type ne 'text/html' ) {
    $rc = 1402;
    &FromDualMySQLagent::log($main::gParameter{'LogFile'}, ERR, "      Weird content type at " . $main::gParameter{'Url'} . ": " . $response->content_type . " (rc=$rc)");
    return $rc;
  }

  # Data loaded
  if ( $response->content =~ /Data loaded successful/ ) {
    if ( $main::gParameter{'Debug'} == DBG ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, DBG, "Data loaded."); }
  }
  # Data load failed
  else {
    $rc = 1403;
    &FromDualMySQLagent::log($main::gParameter{'LogFile'}, ERR, "      Data upload failed. (rc=$rc)");
    &FromDualMySQLagent::log($main::gParameter{'LogFile'}, ERR, '      ' . $response->content);
    return $rc;
  }

  if ( $main::gParameter{'Debug'} >= INFO ) { &FromDualMySQLagent::log($main::gParameter{'LogFile'}, INFO, "      Data uploaded successfully (rc=$rc)."); }
  &FromDualMySQLagent::clearCacheFile($main::gParameter{'CacheFile'});

  return $rc;
}

1;
__END__
